.\" file MASS/lqs.d
.\" copyright (C) 1998, 9 B. D. Ripley
.\"
.BG
.FN lqs
.TL
Resistant Regression
.DN
Fit a regression to the `good' points in the dataset, thereby achieving
a regression estimator with a high breakdown point.
.CS
lqs(x, ...)
lqs.formula(formula, data = NULL, ...,
            method = c("lts", "lqs", "lms", "S", "model.frame"),
            subset, na.action = na.fail, model = TRUE, 
            x = FALSE, y = FALSE, contrasts = NULL)
lqs.default(x, y, intercept, method = c("lts", "lqs", "lms", "S"),
            quantile, control = lqs.control(...), k0 = 1.548, seed, ...)
.RA
.AG formula
a formula of the form `y ~ x1 + x2 + ...{}'.
.AG data
data frame from which variables specified in `formula' are
preferentially to be taken.
.AG subset
An index vector specifying the cases to be used in fitting. (NOTE: If
given, this argument must be named exactly.)
.AG na.action
A function to specify the action to be taken if `NA's are found. The
default action is for the procedure to fail. An alternative is
`na.omit', which leads to omission of cases with missing values on any
required variable.  (NOTE: If given, this argument must be named
exactly.)
.AG x
a matrix or data frame containing the explanatory variables.
.AG y
the response: a vector of length the number of rows of `x'.
.AG intercept
should the model include an intercept?
.AG method
the method to be used. `model.frame' returns the model frame: for the
others see the `Details' section.
.AG quantile
the quantile to be used: see `Details'. This is over-ridden if 
`method = "lms"'.
.AG control
additional control items: see `Details'.
.AG seed
the seed to be used for random sampling: see `.Random.seed'. The
current value of `.Random.seed' will be preserved if it is set..
.AG ...
arguments to be passed to `lqs.default' or `lqs.control'.
.RT
An object of class `"lqs"'.
.DT
Suppose there are `n' data points and `p' regressors, including any
intercept.
.PP
The first three methods minimize some function of the sorted squared
residuals. For methods `"lqs"' and `"lms"' is the `quantile' squared
residual, and for `"lts"' it is the sum of the `quantile' smallest
squared residuals. `"lqs"' and `"lms"' differ in the defaults for
`quantile', which are `floor((n+p+1)/2)' and `floor((n+1)/2)'
respectively. For `"lts"' the default is `floor(n/2) +
floor((p+1)/2)'.
.PP
The `"S"' estimation method solves for the scale `s' such that the
average of a function chi of the residuals divided by `s' is equal to
a given constant.
.PP  
The `control' argument is a list with components
.RC psamp
the size of each sample. Defaults to `p'.
.RC nsamp
the number of samples or `"best"' or `"exact"' or `"sample"'. If
`"sample"' the number chosen is `min(5*p, 3000)', taken from Rousseeuw
and Hubert (1997). If `"best"' exhaustive enumeration is done up to
5000 samples: if `"exact"' exhaustive enumeration will be attempted
however many samples are needed.
.RC adjust
should the intercept be optimized for each sample?
.SH NOTE
There seems no reason other than historical to use the `lms' and `lqs'
options.  LMS estimation is of low efficiency (converging at rate
n^{-1/3}) whereas LTS has the same asymptotic efficiency as an M
estimator with trimming at the quartiles (Marazzi, 1993, p.201).  LQS
and LTS have the same maximal breakdown value of 
`(floor((n-p)/2) + 1)/n' attained if 
`floor((n+p)/2) <= quantile <= floor((n+p+1)/2)'. 
The only drawback mentioned of LTS is greater
computation, as a sort was thought to be required (Marazzi, 1993,
p.201) but this is not true as a partial sort can be used (and is used
in this implementation).
.PP
Adjusting the intercept for each trial fit does need the residuals to
be sorted, and may be significant extra computation if `n' is large
and `p' small.
.PP
Opinions differ over the choice of `psamp'. Rousseeuw and Hubert
(1997) only consider p; Marazzi (1993) recommends p+1 and suggests
that more samples are better than adjustment for a given computational
limit.
.PP
The computations are exact for a model with just an intercept and
adjustment, and for LQS for a model with an intercept plus one
regressor and exhaustive search with adjustment. For all other cases
the minimization is only known to be approximate.
.SH REFERENCES
P. J. Rousseeuw and A. M. Leroy (1987) 
.ul
Robust Regression and Outlier Detection.
Wiley.
.PP
A. Marazzi (1993) 
.ul
Algorithms, Routines and S Functions for Robust Statistics.
Wadsworth & Brooks/Cole.
.PP 
P. Rousseeuw and M. Hubert (1997) Recent developments in PROGRESS. In
.ul
L1-Statistical Procedures and Related Topics 
ed Y. Dodge, IMS Lecture Notes volume \fB31\fP, pp. 201-214.
.SA
`lmsreg', `ltsreg'
.EX
stackloss <- data.frame(stack.x, stack.loss)
lqs(stack.loss ~ ., data=stackloss)
lqs(stack.loss ~ ., data=stackloss, method="S", nsamp="exact")
.KW model
.KW robust
.WR
