.BG
.FN trls.influence
.TL
Regression diagnostics for trend surfaces
.DN
This function provides the basic quantities which are used in
forming a variety of diagnostics for checking the quality of
regression fits for trend surfaces calculated by `surf.ls'.
.CS
trls.influence(object)
plot.trls(x, border = 1, col = NA, pch = 4, cex = 0.6, 
          add = F, div = 8, ...)
.RA
.AG object, x
Fitted trend surface model from `surf.ls'
.OA
.AG div
scaling factor for influence circle radii in `plot.trls'
.AG add
add influence plot to existing graphics if TRUE
.AG border, col, pch, cex, ...
additional graphical parameters
.RT
`trls.influence' returns a list with:
.AG r
raw residuals as given by `residuals.trls'
.AG hii
diagonal elements of the Hat matrix
.AG stresid
standardised residuals
.AG Di
Cook's statistic
.SH REFERENCES
Unwin, D. J., Wrigley, N. (1987) Towards a general-theory of
control point distribution effects in trend surface models. 
.ul
Computers and Geosciences,
\fB13\fR, 351-355.
.SA
`surf.ls', `influence.measures', `plot.lm'
.EX
library(MASS)
topo2 <- surf.ls(2, topo)
infl.topo2 <- trls.influence(topo2)
cand <- as.data.frame(infl.topo2)[abs(infl.topo2$stresid) > 1.5,]
cand
cand.xy <- topo[as.integer(dimnames(cand)[[1]]), c("x", "y")]
trsurf <- trmat(topo2, 0, 6.5, 0, 6.5, 50)
eqscplot(trsurf, type="n")
#under S need to choose appropriate colour numbers
contour(trsurf, add=T, col="grey")
plot(topo2, add=T, div=3)
points(cand.xy, pch=16, col="orange")
.KW spatial
.WR
