.\" file treefix/prune.tree.d based on
.\" @(#)prune.tr.d version 3.15 created 8/6/93
.\" @(#)Copyright (c), 1987, 1993 StatSci, Inc.  All rights reserved.
.\" Amended by B.D. Ripley, changes (C) 1996-9 B. D. Ripley
.BG
.FN prune.tree
.FN prune.misclass
.TL
Cost-complexity Pruning of Tree Object
.DN
Determines a nested sequence of subtrees of the supplied tree by recursively
"snipping" off the least important splits, based upon the cost-complexity
measure.  `prune.misclass' is an abbreviation for
`prune.tree(method = "misclass")' for use with `cv.tree'.
.PP
If `k' is supplied, the optimal subtree is returned.
.CS
prune.tree(tree, k = NULL, best = NULL, newdata, nwts,
        method = c("deviance", "misclass")[1], loss, eps=1e-3)
prune.misclass(tree, k = NULL, best = NULL, newdata, 
        nwts, loss, eps=1e-3)
.RA
.AG tree
fitted model object of class `tree'.  This is assumed to be the result
of some function that produces an object with the same named
components as that returned by the `tree()' function.
.OA
.AG k
cost-complexity parameter defining either a specific subtree of `tree'
(`k' a scalar) or the (optional) sequence of subtrees minimizing the
cost-complexity measure (`k' a vector). If missing, `k' is determined
algorithmically.
.AG best
integer requesting the size (i.e. number of terminal nodes) of a
specific subtree in the cost-complexity sequence to be returned. This
is an alternative way to select a subtree than by supplying a scalar
cost-complexity parameter `k'.  If there is no tree in the sequence of
the requested size, the next largest is returned.
.AG newdata
data frame upon which the sequence of cost-complexity subtrees is
evaluated.  If missing, the data used to grow the tree are used.
.AG nwts
weights for the `newdata' cases.
.AG method
character string denoting the measure of node heterogeneity used to
guide cost-complexity pruning.  For regression trees, only the
default, `deviance', is accepted.  For classification trees, the
default is `deviance' and the alternative is `misclass'
(number of misclassifications or total loss).
.AG loss
a matrix giving for each true class (row) the numeric loss of
predicting the class (column).  The classes should be in the order of
the levels of the response.  It is conventional for a loss matrix to
have a zero diagonal.
.AG eps
a lower bound for the probabilities, used to compute deviances if
events of predicted probability zero occur in `newdata'.
.RT
If `k' is supplied and is a scalar, a `tree' object is returned that
minimizes the cost-complexity measure for that `k'.  If `best' is
supplied, a `tree' object of size `best' is returned.  Otherwise, an
object of class `tree.sequence' is returned.  The object contains the
following components:
.RC size
number of terminal nodes in each tree in the cost-complexity pruning sequence.
.RC deviance
total deviance of each tree in the cost-complexity pruning sequence.
.RC k
the value of the cost-complexity pruning parameter of each tree in 
the sequence.
.DT
The response as well as the predictors referred to in the right side
of the formula in `tree' must be present by name in `newdata'.  These
data are dropped down each tree in the cost-complexity sequence and
deviances or losses calculated by comparing the supplied response to
the prediction.  The function `cv.tree()' routinely uses the `newdata'
argument in cross-validating the pruning procedure.  A `plot()' method
exists for objects of this class.  It displays the value of the
deviance, the number of misclassifications or the total loss for each
subtree in the cost-complexity sequence.  An additional axis displays
the values of the cost-complexity parameter at each subtree.
.EX
#determine the cost complexity pruning sequence
zp <- prune.tree(z.auto)
plot(zp)          #plot it
#select the best 5 node subtree
z5.auto <- prune.tree(z.auto, best = 5)   
.KW tree
.WR
